val versions = Map(
  "hadoop" -> "2.7.3",
  "spark" -> "2.2.0",
  "scala" -> "2.11.8",
  "logback" -> "1.1.9",
  "fastutil" -> "7.0.13",
  "zookeeper" -> "3.4.9",
  "scala-logging" -> "3.5.0",
  "scalatest" -> "3.0.1",
  "junit" -> "4.12",
  "junit-interface" -> "0.11",
  "trove" -> "3.0.3",
  "kryo" -> "3.0.2",
  "scopt" -> "3.5.0"
)

lazy val commonSettings = Seq(
  organization := "snu.datalab",
  version := "0.1.0-SNAPSHOT",
  scalaVersion := "2.11.8",
  fork := true,
  test in assembly := {},
  libraryDependencies ++= Seq(
  ),
  scalacOptions ++= Seq("-optimize", "-Yinline-warnings"),
  packageOptions in assembly ~= { pos =>
    pos.filterNot(_.isInstanceOf[Package.MainClass])
  }
)

lazy val pegasus = (project in file("."))
  .settings(commonSettings: _*)
  .settings(name := "pegasus")
  .aggregate(mapreduce, spark)


lazy val mapreduce = project
  .settings(commonSettings: _*)
  .settings(
    name := "pegasus-mapreduce",
    libraryDependencies ++= Seq(
      "org.apache.hadoop" % "hadoop-client" % versions("hadoop") % "provided",
      "net.sf.trove4j" % "trove4j" % versions("trove"),
      "it.unimi.dsi" % "fastutil" % versions("fastutil"),
      "org.apache.zookeeper" % "zookeeper" % versions("zookeeper"),
      "com.esotericsoftware" % "kryo" % versions("kryo"),
      "junit" % "junit" % versions("junit") % "test",
      "com.novocode" % "junit-interface" % versions("junit-interface") % "test"
    ),
    autoScalaLibrary := false,
    crossPaths := false,
    target in assembly := baseDirectory.value / ".." / "bin",
    testOptions in Test := Seq(Tests.Argument(TestFrameworks.JUnit, "-a", "-v"))
  )

lazy val spark = project
  .settings(commonSettings: _*)
  .settings(
    name := "pegasus-spark",
    libraryDependencies ++= Seq(
      "com.typesafe.scala-logging" %% "scala-logging" % versions("scala-logging"),
      "org.apache.spark" %% "spark-core" % versions("spark") % "provided",
      "org.apache.spark" %% "spark-sql" % versions("spark") % "provided",
      "org.apache.spark" % "spark-graphx_2.11" % "2.2.0" % "provided",
      "it.unimi.dsi" % "fastutil" % versions("fastutil"),
      "org.scalatest" %% "scalatest" % versions("scalatest") % "test",
      "com.github.scopt" %% "scopt" % versions("scopt"),
      "org.apache.commons" % "commons-math3" % "3.6.1",
      "org.sameersingh.scalaplot" % "scalaplot" % "0.0.4"
    ),
    target in assembly := baseDirectory.value / ".." / "bin"
  )
