#!/usr/bin/env bash

# get current path
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

# get binary path of PegasusG on Hadoop
BINARY_NAME=pegasus-spark-assembly-0.1.0-SNAPSHOT.jar
BINARY_PATH=$DIR/../spark/target/$BINARY_NAME

# path on local
GRAPH_DIR=$DIR/graphs
TOOLS_DIR=$DIR/../tools

# paths on HDFS
GRAPH_ON_HDFS=pegasusg-graph
CC_ON_HDFS=pegasusg-cc

# check whether binary of PegasusG on Hadoop exists or not
if [ ! -f "$DIR/../spark/target/$BINARY_NAME" ]; then
  echo "PegasusG binary on Hadoop not found! Build the binary..."
  $TOOLS_DIR/sbt clean assembly
fi

hdfs dfs -rm -r $GRAPH_ON_HDFS $CC_ON_HDFS
hdfs dfs -put $GRAPH_DIR/epinions.tsv $GRAPH_ON_HDFS

# run pagerank
spark-submit --num-executors 3 --executor-memory 1g --driver-memory 1g $BINARY_PATH \
  --class pegasus.spark.cc.PACCExternal \
  -i $GRAPH_ON_HDFS -o $CC_ON_HDFS -t 100 -p 3

# clear unused files
hdfs dfs -rm -r $GRAPH_ON_HDFS
