package pegasus.mapreduce.cc.old;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;

public class PACC extends Configured implements Tool{
	
	public static void main(String[] args) throws Exception{
		ToolRunner.run(new PACC(), args);
	}

	public int run(String[] args) throws Exception{
		
		Configuration conf = getConf();
		
		Path input = new Path(args[0]);
		Path output = new Path(args[1]);
		boolean verbose = conf.getBoolean("verbose", false);
		int numReduceTasks = conf.getInt("mapred.reduce.tasks", 1);
		int numPartitions = conf.getInt("numPartitions", numReduceTasks);
		long localThreshold = conf.getLong("localThreshold", 1000000);
		conf.setInt("numPartitions", numPartitions);
		conf.setLong("mapred.task.timeout", 0L);
		
//		System.out.print("[PACC-start]\t" + input + "\t" + output + "\t" + numPartitions + "\t" + numReduceTasks + "\t" + localThreshold + "\t");
//		System.out.println("# input output numPartitions numReduceTasks localThreshold");
		
		FileSystem fs = FileSystem.get(conf);
		
		fs.delete(output, true);
		
		long time = System.currentTimeMillis();
		long time1, time2;
		long totalTime = time;
		
//		System.out.print("[round 0]\tinit\t");
		Initialization init = new Initialization(input, output.suffix("_0/out"), verbose);
		
		ToolRunner.run(conf, init, null);
		
//		System.out.print( ((System.currentTimeMillis() - time)/1000.0) + "\t" );
//		System.out.println("# time(sec)");
		
		PALargeStarOpt largeStar = null;
		PASmallStarOpt smallStar = null;
		
		long numEdges = init.outputSize;
		long numChanges = 0;
		boolean converge = false;
		int i=0;
		
		do{
//			System.out.printf("[round %d]\t", i+1);
			
			if(numEdges > localThreshold){
//				System.out.print("star\t");
				
				time1 = System.currentTimeMillis();

				largeStar = new PALargeStarOpt(output.suffix("_" + i + "/out"), output.suffix("_large_" + i), verbose); 
				ToolRunner.run(conf, largeStar, null);
				fs.delete(output.suffix("_" + i + "/out"), true);
				
//				System.out.print(largeStar.numChanges + "\t" + largeStar.inputSize + "\t" + largeStar.outSize + "\t" + largeStar.ccSize + "\t");
//				System.out.print( ((System.currentTimeMillis() - time1)/1000.0) + "\t" );
				
				time2 = System.currentTimeMillis();
				
				smallStar = new PASmallStarOpt(output.suffix("_large_" + i + "/out"), output.suffix("_" + (i + 1)), verbose); 
				ToolRunner.run(conf, smallStar, null);
				fs.delete(output.suffix("_large_" + i + "/out"), true);
				
//				System.out.print(smallStar.numChanges + "\t" + smallStar.inputSize + "\t" + smallStar.outSize + "\t" + smallStar.inSize + "\t");
//				System.out.print( ((System.currentTimeMillis() - time2)/1000.0) + "\t" );
//				System.out.print( ((System.currentTimeMillis() - time1)/1000.0) + "\t" );

				numChanges = largeStar.numChanges + smallStar.numChanges;
				numEdges = smallStar.outSize;
				
				converge = (numChanges == 0);
				
//				System.out.println("# L.change L.input L.out L.cc L.time(sec) "
//						+ "S.change S.input S.out S.in S.time(sec) "
//						+ "time");
			}
			else{
//				System.out.print("local\t");
				
//				LocalCC lcc = new LocalCC(output.suffix("_" + i + "/out"), output.suffix("_" + (i+1) + "/out"));
				
				UnionFind lcc = new UnionFind(output.suffix("_" + i + "/out"), output.suffix("_" + (i+1) + "/out"));
				
				time1 = System.currentTimeMillis();
				
				ToolRunner.run(conf, lcc, null);
				
//				System.out.print(numEdges + "\t" + lcc.outputSize + "\t");
//				System.out.print( ((System.currentTimeMillis() - time1)/1000.0) + "\t" );
//				System.out.println("# inputSize outputSize time(sec)");
				
				fs.delete(output.suffix("_" + i + "/out"), true);
				
				numEdges = lcc.outputSize;
				numChanges = 0;
				converge = true;
			}
			
			i++;
			
			
		}while(!converge && fs.exists(output.suffix("_" + i + "/out")));
		
		
		
//		System.out.printf("[round %d]\tfinal\t", i+1);
		
		time = System.currentTimeMillis();
		
		Finalization fin = new Finalization(output, i, verbose);
		
		ToolRunner.run(conf, fin, null);
		
//		System.out.print(fin.inputSize + "\t");
//		System.out.print( ((System.currentTimeMillis() - time1)/1000.0) + "\t" );
//		System.out.println("# inputSize time(sec)");
		
		for(int r = 0; r <= i; r++){
			fs.delete(output.suffix("_"+r), true);
			
//			if(fs.exists(output.suffix("_large_"+r+"/final"))){
//				fs.rename(output.suffix("_large_"+r+"/final"), output.suffix("/"+r));
//			}
			
			fs.delete(output.suffix("_large_"+r), true);
		}
		
		System.out.print("[PACC-end]\t" + input + "\t" + output + "\t" + numPartitions + "\t" + numReduceTasks + "\t" + localThreshold + "\t" + (i+1) + "\t");
		System.out.print( ((System.currentTimeMillis() - totalTime)/1000.0) + "\t" );
		System.out.println("# input output numPartitions numReduceTasks localThreshold numRounds time(sec)");
		
		
		return 0;
	}
}
