package pegasus.mapreduce.cc.old;

import gnu.trove.impl.Constants;
import gnu.trove.map.hash.TLongLongHashMap;
import gnu.trove.procedure.TLongProcedure;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.fs.PathFilter;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.util.Tool;

public class UnionFind extends Configured implements Tool {

	TLongLongHashMap parent = new TLongLongHashMap(Constants.DEFAULT_CAPACITY,
			Constants.DEFAULT_LOAD_FACTOR, -1, -1);

	private final Path input;
	private final Path output;
	public long outputSize = 0;

	public UnionFind(Path input, Path output) {
		this.input = input;
		this.output = output;
	}

	public int run(String[] args) throws Exception {

		Configuration conf = getConf();
		FileSystem fs = FileSystem.get(conf);

		fs.delete(output, true);

		LongWritable iKey = new LongWritable();
		LongWritable iValue = new LongWritable();

		FileStatus[] status = fs.listStatus(input, new PathFilter() {
			public boolean accept(Path path) {
				return path.getName().startsWith("part");
			}
		});

		for(int i = 0; i < status.length; i++){
			try{
				SequenceFile.Reader sr = new SequenceFile.Reader(fs, status[i].getPath(), conf);

				while(sr.next(iKey, iValue)){
					long u = iKey.get();
					long v = iValue.get();

					if(find(u) != find(v)){
						union(u, v);
					}
				}
				sr.close();

			} catch (Exception e){}
		}

		final SequenceFile.Writer out = new SequenceFile.Writer(fs, conf, output,
				LongWritable.class, LongWritable.class);

		final LongWritable ou = new LongWritable();
		final LongWritable ov = new LongWritable();

		parent.forEachKey(new TLongProcedure() {

			public boolean execute(long u) {
				try{
					long v = find(u);
					if(u != v){
						ou.set(u);
						ov.set(v);
						out.append(ou, ov);
						outputSize++;
					}
				} catch (IOException e){}
				return true;
			}
		});

		out.close();

		return 0;
	}

	public long find(long x) {

		long p = parent.get(x);
		if(p != -1){
			long new_p = find(p);
			parent.put(x, new_p);
			return new_p;
		}
		else{
			return x;
		}

	}

	public void union(long x, long y) {
		long r1 = find(x);
		long r2 = find(y);

		if(r1 > r2){
			parent.put(r1, r2);
		}
		else if(r1 < r2){
			parent.put(r2, r1);
		}
	}
}
