package pegasus.mapreduce.common.cache;

import org.apache.hadoop.io.Writable;

public abstract class BaseVectorCache<V extends Writable> implements VectorCache<V> {
  private BitSet containsSet;
  protected int size;
  protected int numBlocks;
  protected long numVertices;
  protected int blockId;

  public BaseVectorCache(final int numBlocks, final int blockId, final long numVertices) {
    this.numBlocks = numBlocks;
    this.numVertices = numVertices;
    this.blockId = blockId;
    this.size = (int) Math.ceil(numVertices / (double) numBlocks);
    this.containsSet = new BitSet(size);
  }

  @Override
  public int size() {
    return size;
  }

  @Override
  public boolean contains(final int i) {
    return containsSet.contains(i);
  }

  @Override
  public void put(int i, V value) {
    containsSet.add(i);
  }

  @Override
  public void addTo(int i, V value) {
    containsSet.add(i);
  }

  @Override
  public void clear() {
    containsSet.clear();
  }
}
