package pegasus.mapreduce.common.cache;

import java.util.Arrays;

public final class BitSet {
  private static final long[] MASKS;
  private final long[] bits;
  private final int cardinality;

  public BitSet(final int cardinality) {
    final int adjustedSize = (int) Math.ceil(cardinality / 64.0);
    this.cardinality = cardinality;
    this.bits = new long[adjustedSize];
  }

  public final int getCardinality() {
    return cardinality;
  }

  public final int getArraySize() {
    return bits.length;
  }

  public final boolean contains(final int id) {
    final int pos = id / 64;
    final long mask = MASKS[id % 64];

    return (bits[pos] & mask) != 0;
  }

  public final void add(final int id) {
    final int pos = id / 64;
    final long mask = MASKS[id % 64];

    bits[pos] |= mask;
  }

  public final void remove(final int id) {
    final int pos = id / 64;
    final long mask = MASKS[id % 64];

    bits[pos] &= ~mask;
  }

  public final void clear() {
    Arrays.fill(bits, 0L);
  }

  static {
    MASKS = new long[64];
    for (int i = 0; i < 64; ++i) {
      MASKS[i] = 1L << i;
    }
  }
}
