package pegasus.mapreduce.common.io;

import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.LineRecordReader;

import java.io.IOException;

public class ContinuousDistributionInputFormat extends FileInputFormat<LongWritable, DoubleWritable> {
  @Override
  public RecordReader<LongWritable, DoubleWritable> createRecordReader(InputSplit split, TaskAttemptContext context)
      throws IOException, InterruptedException {
    return new DistributionRecordReader();
  }

  private static class DistributionRecordReader extends RecordReader<LongWritable, DoubleWritable> {
    private final LineRecordReader reader = new LineRecordReader("\n".getBytes());
    private final LongWritable key = new LongWritable();
    private final DoubleWritable value = new DoubleWritable();

    @Override
    public void initialize(InputSplit split, TaskAttemptContext context) throws IOException, InterruptedException {
      reader.initialize(split, context);
    }

    @Override
    public boolean nextKeyValue() throws IOException, InterruptedException {
      if (reader.nextKeyValue()) {
        final Text line = reader.getCurrentValue();
        final String lineStr = line.toString();
        final int split = line.find("\t");
        key.set(Long.valueOf(lineStr.substring(0, split)));
        value.set(Double.valueOf(lineStr.substring(split + 1)));

        return true;
      }
      return false;
    }

    @Override
    public LongWritable getCurrentKey() throws IOException, InterruptedException {
      return key;
    }

    @Override
    public DoubleWritable getCurrentValue() throws IOException, InterruptedException {
      return value;
    }

    @Override
    public float getProgress() throws IOException, InterruptedException {
      return reader.getProgress();
    }

    @Override
    public void close() throws IOException {
      reader.close();
    }
  }
}
