/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Ha-Myung Park, Chiwan Park, U Kang
 *
 * This software is licensed under Apache License, Version 2.0 (the  "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * -------------------------------------------------------------------------
 * File: IDUtils.java
 * - Provides ID conversion methods between local vertex ID and global vertex ID.
 * Version: 3.0
 */
package pegasus.mapreduce.common.utils;

/**
 * Provides some utility methods about vertex IDs.
 */
public class IDUtils {
  /**
   * Converts a global vertex ID to local vertex ID.
   *
   * @param numBlocks The number of total vector blocks
   * @param globalId  A global ID of vertex
   * @return A local ID of the given vertex
   */
  public static int globalToLocal(int numBlocks, long globalId) {
    return (int) (globalId / numBlocks);
  }

  /**
   * Converts a local vertex ID to global vertex ID.
   *
   * @param numBlocks The number of total vector blocks
   * @param blockId   An ID of vector block who contains the vertex
   * @param localId   A local ID of vertex
   * @return A global ID of the given vertex
   */
  public static long localToGlobal(int numBlocks, int blockId, int localId) {
    return ((long) localId) * numBlocks + blockId;
  }
}
