/*
 * PegasusN: Peta-Scale Graph Mining System (Pegasus v3.0)
 * Authors: Ha-Myung Park, Chiwan Park, U Kang
 *
 * This software is licensed under Apache License, Version 2.0 (the  "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * -------------------------------------------------------------------------
 * File: MemoryUtils.java
 * - Provides memory utility methods to inspect current memory status.
 * Version: 3.0
 */
package pegasus.mapreduce.common.utils;

/**
 * Provides some memory utility methods to inspect current memory status.
 */
public class MemoryUtils {
  private static final Runtime JavaRuntime = Runtime.getRuntime();

  /**
   * Call garbage collector 5 times explicitly.
   */
  public static void callGCMultipleTimes() {
    callGCMultipleTimes(5);
  }

  /**
   * Call garbage collector explicitly.
   *
   * @param times The number of calling garbage collector
   */
  public static void callGCMultipleTimes(int times) {
    for (int i = 0; i < times; ++i) {
      JavaRuntime.gc();
    }
  }

  /**
   * Returns the maximum heap size in bytes.
   *
   * @return The maximum heap size in bytes
   */
  public static long maxMemoryBytes() {
    return JavaRuntime.maxMemory();
  }

  /**
   * Returns the available heap size in bytes.
   *
   * @return The available heap size in bytes.
   * @see <a href="https://stackoverflow.com/a/12807848/2034780">https://stackoverflow.com/a/12807848/2034780</a>
   */
  public static long availableMemoryBytes() {
    return JavaRuntime.maxMemory() - JavaRuntime.totalMemory() + JavaRuntime.freeMemory();
  }

  /**
   * Converts the given memory size from bytes to mebibytes.
   *
   * @param bytes A size of memory in bytes
   * @return The size of memory in mebibytes
   */
  public static long bytesToMebibytes(final long bytes) {
    return (long) (bytes / 1024.0 / 1024.0);
  }
}
