package pegasus.mapreduce.common.utils;

import org.apache.zookeeper.CreateMode;
import org.apache.zookeeper.KeeperException;
import org.apache.zookeeper.ZooDefs;
import org.apache.zookeeper.ZooKeeper;

import java.util.ArrayList;
import java.util.List;

public class ZookeeperUtils {
  public static void createPersistentNode(final ZooKeeper zk, final String path) {
    createNode(zk, path, CreateMode.PERSISTENT);
  }

  public static void createOrUpdatePersistentNode(final ZooKeeper zk, final String path, final String value) {
    final byte[] valueBytes = value.getBytes();
    if (exists(zk, path)) {
      try {
        zk.setData(path, valueBytes, -1);
      } catch (KeeperException | InterruptedException e) {
        // ignore
      }
    } else {
      createNode(zk, path, valueBytes, CreateMode.PERSISTENT);
    }
  }

  public static String getNodeData(final ZooKeeper zk, final String path) {
    try {
      return new String(zk.getData(path, false, null));
    } catch (KeeperException | InterruptedException e) {
      return "";
    }
  }

  public static void createEphemeralNode(final ZooKeeper zk, final String path) {
    createNode(zk, path, CreateMode.EPHEMERAL);
  }

  private static void createNode(final ZooKeeper zk, final String path, final CreateMode createMode) {
    createNode(zk, path, new byte[0], createMode);
  }

  private static void createNode(
      final ZooKeeper zk, final String path, final byte[] values, final CreateMode createMode) {
    try {
      zk.create(path, values, ZooDefs.Ids.OPEN_ACL_UNSAFE, createMode);
    } catch (KeeperException e) {
      switch (e.code()) {
        case NODEEXISTS:
          break;
        default:
          break;
      }
    } catch (InterruptedException e) {
      // ignore
    }
  }

  public static boolean exists(final ZooKeeper zk, final String path) {
    try {
      return zk.exists(path, false) != null;
    } catch (KeeperException | InterruptedException e) {
      return false;
    }
  }

  public static List<String> getChildren(final ZooKeeper zk, final String path) {
    try {
      return zk.getChildren(path, false);
    } catch (KeeperException | InterruptedException e) {
      return new ArrayList<>();
    }
  }

  public static void delete(final ZooKeeper zk, final String path) {
    try {
      zk.delete(path, -1);
    } catch (KeeperException | InterruptedException ignored) {
    }
  }
}
