package pegasus.mapreduce.triangle.deprecated;

import com.esotericsoftware.kryo.KryoException;
import com.esotericsoftware.kryo.io.Input;
import gnu.trove.list.array.TIntArrayList;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.lib.input.NLineInputFormat;
import org.apache.hadoop.mapreduce.lib.output.NullOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;
import org.xerial.snappy.SnappyInputStream;
import pegasus.mapreduce.triangle.PrePartition;

import java.io.IOException;
import java.util.*;

public class PTE_cd extends Configured implements Tool{

	long SHUFFLED_BYTES = 0;
	long HDFS_BYTES_READ = 0;
	long TRIANGLES = 0;
	long OPERATIONS = 0;
	
	public static void main(String args[]) throws Exception{
		ToolRunner.run(new PTE_cd(), args);
	}
	
	public int run(String[] args) throws Exception{
		
		long pt, et;
		
		if(args.length < 1){
			System.out.println("parameters: inputFilePath numColors");
			return 0;
		}
		
		String input = args[0];
		
		String part = input + ".part";
		String seed = input + ".seed";
		
		int numColors = Integer.parseInt(args[1]);
		
		pt = System.currentTimeMillis();
		
		PrePartition pp = new PrePartition();
		pp.setConf(getConf());

		pp.partitionGraph(input, part, numColors);
		int nodeSize = pp.nodeSize;
		System.out.println("nodeSize: " + nodeSize);
		
		pt = (System.currentTimeMillis() - pt)/1000;

		createSeedFile(seed, numColors);
		
		et = System.currentTimeMillis();
		enumerateTriangles(numColors, nodeSize, part, seed);
		et = (System.currentTimeMillis() - et)/1000;
		
		System.out.println("partitionTime\tEnumerationTime\tShuffle\tNetwork\tTriangles\tOperations");
		System.out.println("[" + this.getClass().getSimpleName() + "]" + input + "\t" + pt + "\t"
				+ et + "\t" + SHUFFLED_BYTES + "\t" + HDFS_BYTES_READ + "\t" + TRIANGLES + "\t"
				+ OPERATIONS);
		
		FileSystem.get(getConf()).delete(new Path(part), true);
		FileSystem.get(getConf()).delete(new Path(seed), true);
		
		return 0;
	}
	
	private void enumerateTriangles(int numColors, int nodeSize, String part, String seed) throws IOException,
			InterruptedException, ClassNotFoundException{
		Job job = Job.getInstance(getConf(), "[" + this.getClass().getSimpleName() + "_TE]" + seed);
		Configuration conf = job.getConfiguration();
		conf.set("mapreduce.map.java.opts", conf.get("mapreduce.map.java.opts", "") +
				" -XX:+UseG1GC -XX:+UseLargePagesInMetaspace");
		
		job.setJarByClass(getClass());
		job.setMapperClass(TriangleEnumerationMapper.class);
		job.setOutputFormatClass(NullOutputFormat.class);
		job.setNumReduceTasks(0);
		conf.setInt("mapred.line.input.format.linespermap", 1);
		conf.set("part", part);
		conf.setInt("numColors", numColors);
		conf.setInt("nodeSize", nodeSize);
		job.setInputFormatClass(NLineInputFormat.class);
		NLineInputFormat.setInputPaths(job, new Path(seed));

		job.getConfiguration().setLong("mapred.task.timeout", 0L);
		job.getConfiguration().setLong("mapred.min.split.size", Long.MAX_VALUE);
		
		job.waitForCompletion(false);
		
		HDFS_BYTES_READ = job.getCounters().findCounter("FileSystemCounters", "HDFS_BYTES_READ").getValue();
		TRIANGLES = job.getCounters().findCounter("PTE", "triangles").getValue();
		OPERATIONS = job.getCounters().findCounter("PTE", "operations").getValue();
		
		System.err.println("triangles : " + job.getCounters().findCounter("PTE", "triangles").getValue());
		System.err.println("operations: " + job.getCounters().findCounter("PTE", "operations").getValue());
	}

	private String createSeedFile(String seedFile, int p) throws IOException{
		
		Configuration conf = getConf();
		
		List<String> seedList = new ArrayList<>();
		
		FSDataOutputStream seedOut = FileSystem.get(conf).create(new Path(seedFile));
		
		for(int i=0; i < p-2; i++){
			for(int j=i+1; j < p-1; j++){
				for(int k=j+1; k < p; k++){
					
					seedList.add(String.format("%10d %10d %10d\n", i,j,k));
				}
			}
		}
		
		for(int i=0; i < p-1; i++){
			for(int j=i+1; j < p; j++){
				seedList.add(String.format("%10d %10d\n", i,j));
			}
		}
		
		Collections.shuffle(seedList);
		
		System.out.printf("[Info] %d subproblem(s) to process\n", seedList.size());
		for(String s : seedList){
			seedOut.writeBytes(s);
		}
		
		seedOut.close();
		
		return seedFile;
		
	}

	public static class TriangleEnumerationMapper extends Mapper<Object, Text, NullWritable, NullWritable>{

		int numColors;
		int nodeSize;
		Configuration conf;
		FileSystem fs;
		Path part;
		
		
		long operationCounter = 0;

		@Override
		protected void setup(Context context)
				throws IOException, InterruptedException{
			conf = context.getConfiguration();
			numColors = conf.getInt("numColors", 0);
			nodeSize = context.getConfiguration().getInt("nodeSize", 0);
			fs = FileSystem.get(conf);
			part = new Path(conf.get("part"));
		}
		
		@Override
		protected void map(Object key, Text value,
				Context context)
				throws IOException, InterruptedException{
			
			int[] problem = getProblem(value);
			System.out.println(Arrays.toString(problem) + "-subproblem");
			long count = 0;
			operationCounter = 0;
			
			if(problem.length == 2){
				//(i,j)-subproblem
				
				System.out.printf("free memory: %.2fMB\n", availableMemoryBytes(true) / 1000000.0);
				
				int[][] ii = getAdj(part.suffix("/edgeset-" + (problem[0] * numColors + problem[0])));
				int[][] ij = getAdj(part.suffix("/edgeset-" + (problem[0] * numColors + problem[1])));
				int[][] ji = getAdj(part.suffix("/edgeset-" + (problem[1] * numColors + problem[0])));
				int[][] jj = getAdj(part.suffix("/edgeset-" + (problem[1] * numColors + problem[1])));
				
				if(problem[0] + 1 == problem[1]){
					count += countTriangles(ii, ii, ii);
				}
				
				if(problem[1] == numColors-1 && problem[0] == 0){
					count += countTriangles(jj, jj, jj);
				}
				
				count += countTriangles(ii,ij,ij);
				count += countTriangles(ij,ii,ji);
				count += countTriangles(ij,ij,jj);
				count += countTriangles(ji,ji,ii);
				count += countTriangles(ji,jj,ij);
				count += countTriangles(jj,ji,ji);
				
				
			}
			else{
				//(i,j,k)-subproblem
				
				System.out.printf("free memory: %.2fMB\n", availableMemoryBytes(true) / 1000000.0);
				
				int[][] ij = getAdj(part.suffix("/edgeset-" + (problem[0] * numColors + problem[1])));
				int[][] ik = getAdj(part.suffix("/edgeset-" + (problem[0] * numColors + problem[2])));
				int[][] jk = getAdj(part.suffix("/edgeset-" + (problem[1] * numColors + problem[2])));
				int[][] kj = getAdj(part.suffix("/edgeset-" + (problem[2] * numColors + problem[1])));
				int[][] ji = getAdj(part.suffix("/edgeset-" + (problem[1] * numColors + problem[0])));
				int[][] ki = getAdj(part.suffix("/edgeset-" + (problem[2] * numColors + problem[0])));

				count += countTriangles(ij,ik,jk);
				count += countTriangles(ik,ij,kj);
				count += countTriangles(ji,jk,ik);
				count += countTriangles(jk,ji,ki);
				count += countTriangles(ki,kj,ij);
				count += countTriangles(kj,ki,ji);

			}


			System.out.println("count: " + count);
			context.getCounter("PTE", "triangles").increment(count);
			
		}

		private int[] getProblem(Text line){
			StringTokenizer st = new StringTokenizer(line.toString());

			TIntArrayList _problem = new TIntArrayList();

			try{
				while(true){
					_problem.add(Integer.parseInt(st.nextToken()));
				}
			}catch(Exception e){}

			int[] problem = _problem.toArray();

			return problem;
		}
		
		private int[][] getAdj(Path graphPath) throws IOException{
			long t = System.currentTimeMillis();

			int[][] adj = new int[nodeSize][];

//			DataInputStream in = new DataInputStream(new SnappyInputStream(fs.open(graphPath).getWrappedStream()));
			
			Input in = new Input(new SnappyInputStream(fs.open(graphPath).getWrappedStream()));
			
			try{
				while(true){
					int u = in.readInt(true);
					int uSize = in.readInt(true);
					
					adj[u] = new int[uSize];
					int[] uAdj = adj[u];
					
					for(int i=0; i<uSize; i++){
						uAdj[i] = in.readInt(true);
					}
				}
			}catch(KryoException e){}
			
			in.close();
			
			System.out.printf("loading[%s]: %.3fs (free memory: %.2fMB)\n",
					graphPath.toString(),
					(System.currentTimeMillis() - t) / 1000.0,
					availableMemoryBytes(true) / 1000000.0);
			
			return adj;
		}
		
		private long countTriangles(int[][] ij, int[][] ik, int[][] jk){
			
			long t = System.currentTimeMillis();
			
			long count = 0;
			
			for(int u = 0; u < nodeSize; u++){
				if(ij[u] != null){
					for(int v : ij[u]){
							count += countIntersect(ik[u], jk[v]);
					}
				}
			}
			
			System.out.println("counting: " + ((System.currentTimeMillis() - t)/1000.0) + "s");
			
			return count;
			
		}
		
		private long countIntersect(int[] uAdj, int[] vAdj){

			if(uAdj == null || vAdj == null){
				return 0;
			}

			long count = 0;
			int uCur = 0;
			int vCur = 0;
			long uD = uAdj.length;
			long vD = vAdj.length;

			while(uCur < uD && vCur < vD){
				if(uAdj[uCur] < vAdj[vCur]) uCur++;
				else if(vAdj[vCur] < uAdj[uCur]) vCur++;
				else{
					count++;
					uCur++;
					vCur++;
				}
			}

			return count;
		}
		
		public static long availableMemoryBytes(boolean gc){
			
			if(gc){
				System.gc(); System.gc();
			}
			
			return Runtime.getRuntime().maxMemory()
			- (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
		}
		
	}
	
}
