package pegasus.spark.subgraph;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import java.util.Arrays;

public class VF2_degree{

	private final Logger LOGGER = Logger.getLogger(this.getClass().getName());

	/* data graph variables */
	int[][] adj; // data graph.


	/* pattern graph variables */
	boolean[][] pattern; //pattern[i][j] = pattern[j][i] = true if i and j are linked.
	boolean[][] partialOrder; // partialOrder[i][j] = true if i < j.
	int[][] fr; //for checking feasibility rule

	/* match state variables */
	int[] state;
	int[] match;
	int[] cand_size;
	int[] candidate;
	long[][] dependency; // dependency[i][j] = the minimum possible value of j in i-th level = the maximum node having a dependency relation with j in i-th level.
	// high 32 bits = degree, low 32 bits = node id.
	long count;
	long colorbit;

	/* color related variables */
	long[] filter;
	int numColors;
	int[] colors;
	int numColors_ori;


	private boolean contains(long item, long[] itemSet){
		for(int i=0; i<itemSet.length; i++){
			if(itemSet[i] == item) return true;
		}
		return false;
	}

	public long countSubgraphs(int[][] adj, String patternString, long[] filter, int[] colors, int numColors_ori){

		LOGGER.setLevel(Level.WARN);

		this.adj = adj;
		this.filter = filter;
		this.colors = colors;
		this.numColors = colors.length;
		this.numColors_ori = numColors_ori;

		pattern = parsePatternString(patternString);
		partialOrder = new PartialOrder(pattern).get();
		fr = constructFeasibilityRule(pattern);

		int patternSize = pattern.length;

//		System.out.print("Pattern partial order: ");
//		for(int i=0; i<patternSize; i++){
//			for(int j=0; j<patternSize; j++){
//				if(partialOrder[i][j]) System.out.printf("(%d,%d) ",i,j);
//			}
//		}
//		System.out.println();

		int numNodes = adj.length;

		//initialize match state variables
		match = new int[patternSize];
		cand_size = new int[patternSize];
		dependency = new long[patternSize][patternSize];
		state = new int[numNodes];
		candidate = new int[numNodes];
		count = 0;

		LOGGER.info(String.format("subgraph enumeration start! (free-mem %.2fMB)", availableMemoryBytes(true)/1000000.0));

		int progress = 0;

		for(int u = 0; u < numNodes; u++){

			if(adj[u] == null) continue;
			if(prepare(0, u)){
				vf2(1);
			}
			restore(0, u, 0, 0);

			int progress_cur = u * 10 / numNodes;

			if(progress_cur != progress){
				LOGGER.info("processed nodes:" + u + "/" + numNodes);
				progress = progress_cur;
			}
		}

		return count;
	}


	public boolean[][] parsePatternString(String patternString){

		int patternSize; for (patternSize = 1; patternSize*(patternSize-1)/2 < patternString.length(); patternSize++);
		boolean[][] pattern = new boolean[patternSize][patternSize];
		int k = 0;
		for(int j = 0; j < patternSize; j++){
			for(int i = j + 1; i < patternSize; i++){
				pattern[i][j] = pattern[j][i] = patternString.charAt(k++) == '1';
			}
		}

		return pattern;
	}

	private class PartialOrder{

		private boolean[][] pattern;
		private boolean[][] partialOrder;
		private int[] path;
		private int patternSize;

		public PartialOrder(boolean[][] pattern) {
			this.pattern = pattern;
			this.patternSize = pattern.length;
			this.partialOrder = new boolean[patternSize][patternSize];
			this.path = new int[pattern.length];
		}

		public boolean[][] get(){
			dfs(0);
			return partialOrder;
		}

		private void dfs(int p){
			if(p == path.length){
				for(int i = 0; i < patternSize; i++){
					for(int j = 0; j < patternSize; j++){
						if(pattern[i][j] != pattern[path[i]][path[j]]) return;
					}
				}
				//from here, the path is isomorphic to the original node sequence.
				for (int i = 0; i < patternSize; i++){
					if(path[i] != i){
						partialOrder[i][path[i]] = true;
						return;
					}
				}
			}
			else{
				for(int i = 0; i < patternSize; i++){
					int j = 0;
					for(; j < p; j++) if(path[j] == i) break;
					if(j == p){
						path[p] = i;
						dfs(p+1);
					}
				}
			}
		}

	}

	private int[][] constructFeasibilityRule(boolean[][] pattern) {

		int patternSize = pattern.length;
		int[][] fr = new int[patternSize][3];

		for(int i = 0; i < patternSize; i++){

			for(int j=0; j < i; j++){
				if(pattern[i][j]){
					fr[i][0] |= 1 << j;
				}
			}

			for(int j=i+1; j < patternSize; j++){
				if(pattern[i][j]){
					int k = 0;
					for(; k < i; k++) if(pattern[k][j]) break;
					if(k < i) fr[i][1]++;
					else fr[i][2]++;
				}
			}
		}

		return fr;
	}

	public void vf2(int p){

		int patternSize = pattern.length;

		// nodes are in a total order
		for(int i = 0, lv = 0; lv < p; lv++){
			for(; i < cand_size[lv]; i++){
				int u = candidate[i];

				//safe because nodes are sorted
				long order_u = ((long) adj[u].length << 32) | u;
				if(order_u <= dependency[p-1][p]){
					i = cand_size[lv];
					break;
				}

				// state[u] > 0 means that u is not matched candidate.
				// fr[p][0] == status[u] --> following feasibility rule 1
				if(state[u] > 0 && (fr[p][0] & state[u]) == fr[p][0]){
					if(p+1 >= patternSize){

						if(contains(colorbit | (1 << (u % numColors)), filter)){
							match[p] = u;
							count++;
//							System.out.println(Arrays.toString(match));
						}
					}
					else{
						int t = state[u]; //save state of u
						long t_colorbit = colorbit;


						//check feasibility rule 2 and 3.
						if(prepare(p, u)) vf2(p+1);

						restore(p, u, t, t_colorbit);

					}
				}
			}
		}
	}

	private boolean prepare(int p, int u) {

		match[p] = u; //add u to match
		state[u] = -p-1; //update state of u; u is matched
		cand_size[p] = p == 0 ? 0 : cand_size[p-1]; //set the candidate cursor.
		colorbit |= 1 << (u % numColors); //set colors of matches.

		long m = Long.MAX_VALUE;
		for (int j = p+1; j < pattern.length; j++) {
			dependency[p][j] = p == 0 ? -1 : dependency[p-1][j];

			long order_u = ((long) adj[u].length << 32) | u;

			if (partialOrder[p][j] && dependency[p][j] < order_u)
				dependency[p][j] = order_u;
			if (m > dependency[p][j]) m = dependency[p][j];
		}


		int existCand_cnt = 0, newCand_cnt = 0;
		for(int j = adj[u].length-1; j >= 0; j--){
			int v = adj[u][j];

			//safe because uN is sorted
			long order_v = ((long) adj[v].length << 32) | v;
			if(order_v <= m) break;

			if(state[v] > 0){ // v is candidate but not matched.
				existCand_cnt++;
				state[v] |= 1<<p;
			}
			else if(state[v] == 0){ // v is neither a candidate nor a match.
				newCand_cnt++;
				state[v] |= 1<<p;
				candidate[cand_size[p]++] = v;
			}

		}

		boolean feasibility = existCand_cnt + newCand_cnt >= fr[p][1] + fr[p][2];

//		return feasibility;
		return true;
	}


	private void restore(int p, int u, int t, long t_colorbit) {

		colorbit = t_colorbit; //restore colorbit.
		state[u] = t; // restore status of u.

		long m = Long.MAX_VALUE;
		for (int j = p+1; j < pattern.length; j++) {
			if (m > dependency[p][j]) m = dependency[p][j];
		}

		for(int j = adj[u].length-1; j >= 0; j--){
			int v = adj[u][j];

			long order_v = ((long) adj[v].length << 32) | v;
			if(order_v <= m) break;

			if(state[v] > 0){
				state[v] &= ~(1<<p);
			}

		}

	}

	public static long availableMemoryBytes(boolean gc){

		if(gc){
			System.gc(); System.gc();
		}

		return Runtime.getRuntime().maxMemory()
				- (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory());
	}

}
