package pegasus.spark.cc.deprecated

import java.util.StringTokenizer

import org.apache.log4j.{Level, Logger}
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.rdd.RDD
import org.apache.spark.{Partitioner, SparkConf, SparkContext}
import pegasus.spark.cc.UnionFind
import pegasus.spark.common.io.ExternalSorter

/**
  * Created by hmpark on 17. 3. 16.
  */
object PACC_D {

//  Logger.getLogger("org").setLevel(Level.WARN)
//  Logger.getLogger("akka").setLevel(Level.WARN)

  val logger = Logger.getLogger(getClass)
    logger.setLevel(Level.INFO)

  case class Config(localThreshold: Int = 100000, numPartitions: Int = 80,
                    inputPath: String = "", outputPath: String = "")

  def main(args: Array[String]): Unit = {

    val parser = new scopt.OptionParser[Config]("pacc"){
      head("pacc", "1.0")

      opt[Int]('t', "localThreshold")
        .action( (x, c) => c.copy(localThreshold = x))
        .text("if the number of remaining edges are lower than this value, " +
          "pacc run a single machine algorithm (LocalCC). (default: 100000)")

      opt[Int]('p', "numPartition").required()
        .action((x, c) => c.copy(numPartitions = x))
        .text("the number of partitions. (default: 80)")

      arg[String]("input")
        .action((x, c) => c.copy(inputPath = x))
        .text("input file path.")

      arg[String]("output")
        .action((x,c) => c.copy(outputPath = x))
        .text("output file path.")

    }

    parser.parse(args, Config()) match {
      case Some(opts) =>

        logger.info(f"inputPath: ${opts.inputPath}, output: ${opts.outputPath}, " +
          f"localTrheshold: ${opts.localThreshold}, numPartitions: ${opts.numPartitions}")

        val conf = new SparkConf().setAppName("[PACC]" + opts.inputPath)
          .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
          .set("spark.kryo.registrator", "pegasus.spark.subgraph.MyRegistrator")
          .set("spark.kryoserializer.buffer", "24m")

        val sc = new SparkContext(conf)

        run(opts.inputPath, opts.localThreshold, opts.numPartitions, sc)
          .map{ case (u, v) => u + "\t" + v }
          .saveAsTextFile(opts.outputPath)

      case None =>
    }


  }

  var numPartitions: Broadcast[Int] = _


  def run(inputPath: String, localThreshold: Int, numPartitions: Int, sc: SparkContext): RDD[(Long, Long)] = {

    this.numPartitions = sc.broadcast(numPartitions)

    //initialize
    var out = sc.textFile(inputPath).map{ line =>
      val st = new StringTokenizer(line)
      val u = st.nextToken().toLong
      val v = st.nextToken().toLong
      (u, v)
    }

    var numEdges = out.count()

    var prevRDD = out
    var converge = false

    var cc: RDD[(Long, Long)] = sc.emptyRDD[(Long, Long)]
    var in: RDD[(Long, Long)] = sc.emptyRDD[(Long, Long)]

    do{
      if(numEdges > localThreshold) {
        val (lout, lcc, l_change, lout_size, lcc_size) = largeStar(out)

        lout.foreach(println)

        cc = cc ++ lcc
        val (sout, sin, s_change, sout_size, sin_size) = smallStar(lout)
        out = sout
        in = in ++ sin

        logger.info(f"lout: $lout_size, lcc: $lcc_size, sout: $sout_size, sin: $sin_size")

        converge = l_change == 0 && s_change == 0
        numEdges = sout_size

      }
      else{
        out = UnionFind.run(out.map{ case (u, v) => (if (u < 0) ~u else u, v) })
        converge = true
      }
    }while(!converge)

    ccComputation(out ++ cc ++ in)
  }

  def largeStar(inputRDD: RDD[(Long, Long)]): (RDD[(Long, Long)], RDD[(Long, Long)], Long, Long, Long) = {

    val sc = inputRDD.sparkContext

    val NUM_CHANGES = sc.longAccumulator
    val LCC_SIZE = sc.longAccumulator
    val LIN_SIZE = sc.longAccumulator
    val LOUT_SIZE = sc.longAccumulator

    val groupedRDD = inputRDD.flatMap{
      case (u, v) =>
        if (u < 0) Seq((v, u))
        else Seq((u, v), (v, u))
    }.groupByKey()

    val LOTHERS = true
    val LOUT = false
    val tmpPaths = sc.hadoopConfiguration.getTrimmedStrings("yarn.nodemanager.local-dirs")

    //TODO to implement combiner
    //TODO to replace groupByKey

    val res_all = groupedRDD.flatMap {
      case (u, _uN) =>

        val numPartitions = this.numPartitions.value

        val mcu = Array.fill[Long](numPartitions)(Long.MaxValue)
        mcu((u % numPartitions).toInt) = u

        var isStar = true



        val it_before = _uN.toArray.filter { v_raw =>
          val v = if (v_raw >= 0) {
            isStar = false
            v_raw
          } else ~v_raw

          val vp = (v % numPartitions).toInt
          mcu(vp) = Math.min(v, mcu(vp))

          println(f"($u) mcu_vp: ${mcu(vp)}, vp: $vp, v: $v")

          v > u
        }

        val it = ExternalSorter.sort(it_before.iterator, tmpPaths)

        println(mcu.mkString(","))
        val mu = mcu.min

        if (isStar) it.map { v_raw => LCC_SIZE.add(1); ((LOTHERS, ~v_raw), u) }
        else{
          it.map{ v_raw =>
            val vIsLeaf = v_raw < 0
            val v: Long = if(vIsLeaf) ~v_raw else v_raw

            val vp = (v % numPartitions).toInt
            val mcu_vp = mcu(vp)

            println(f"($u) vp: $vp, numPartitions: $numPartitions, v: $v")
            assert(mcu_vp != Long.MaxValue, f"($u) mcu_vp ($mcu_vp) is wrong........: v=$v")

            if(v != mcu_vp) {

              if(mcu_vp != u) NUM_CHANGES.add(1)

              if(vIsLeaf){
                LIN_SIZE.add(1)
                ((LOTHERS, v), mcu_vp)
              }
              else{
                LOUT_SIZE.add(1)
                ((LOUT, v), mcu_vp)
              }
            }
            else{// v is a local minimum but not the global minimum because 'it' has only large neighbors.
              LOUT_SIZE.add(1)
              ((LOUT, v), mu)
            }
          }
        }

    }

    val partitioner = new CPartitioner(numPartitions.value * 2)

    val res_all_after_partition = res_all.partitionBy(partitioner)

    val lother = new RightHalfRDD(res_all_after_partition)
    val lout = new LeftHalfRDD(res_all_after_partition)

    (lout, lother, NUM_CHANGES.value, LOUT_SIZE.value, LCC_SIZE.value)

  }

  class CPartitioner(numPart: Int) extends Partitioner{
    override def numPartitions: Int = numPart
    override def getPartition(key: Any): Int = {
      val (other, v) = key.asInstanceOf[(Boolean, Long)]
      if(other) (v % numPartitions + numPartitions).toInt
      else (v % numPartitions).toInt
    }
  }

  def smallStar(inputRDD: RDD[(Long, Long)]): (RDD[(Long, Long)], RDD[(Long, Long)], Long, Long, Long) = {

    val sc = inputRDD.sparkContext

    val NUM_CHANGES = sc.longAccumulator
    val SIN_SIZE = sc.longAccumulator
    val SOUT_SIZE = sc.longAccumulator

    val groupedRDD = inputRDD.flatMap{
      case (u, v) => Seq((u, v), (v, u))
    }.groupByKey()

    //TODO to implement combiner
    //TODO to replace groupByKey

    val SIN = true
    val SOUT = false
    val tmpPaths = sc.hadoopConfiguration.getTrimmedStrings("yarn.nodemanager.local-dirs")

    val res_all: RDD[((Boolean, Long), Long)] = groupedRDD.flatMap { case (u, uN) =>

      val numPartitions = this.numPartitions.value

      val mcu = Array.fill[Long](numPartitions)(Long.MaxValue)
      val up = (u % numPartitions).toInt
      mcu(up) = u

      var isLeaf = true

      val it_before = uN.filter { v =>
        if (v > u) isLeaf = false
        val vp = (v % numPartitions).toInt
        mcu(vp) = Math.min(v, mcu(vp))
        v < u
      }

      val it = ExternalSorter.sort(it_before.iterator, tmpPaths)

      val mu = mcu.min

      val sout: Iterator[((Boolean, Long), Long)] = it.filter{_ != mu}.map { v =>

        val vp = (v % numPartitions).toInt

        NUM_CHANGES.add(1)
        SOUT_SIZE.add(1)

        if (v != mcu(vp))
          ((SOUT, v), mcu(vp))
        else  // v is mcu but not mu
          ((SOUT, v), mu)

      }

      if (u != mcu(up)) {
        if (isLeaf) {
          SIN_SIZE.add(1)
          sout ++ Iterator(((SIN, u), mcu(up)))
        }
        else {
          SOUT_SIZE.add(1)
          sout ++ Iterator(((SOUT, u), mcu(up)))
        }
      }
      else if (u != mu) {
        SOUT_SIZE.add(1)
        sout ++ Iterator(((SOUT, if (isLeaf) ~u else u), mu))
      }
      else sout
    }

    val partitioner = new CPartitioner(numPartitions.value * 2)

    val res_all_after_partition = res_all.partitionBy(partitioner)

    val sin = new LeftHalfRDD(res_all_after_partition)
    val sout = new RightHalfRDD(res_all_after_partition)

    (sout, sin, NUM_CHANGES.value, SOUT_SIZE.value, SIN_SIZE.value)

  }

  def ccComputation(remains: RDD[(Long, Long)]): RDD[(Long, Long)] = {

    val numPartitions = this.numPartitions.value

    remains.map{ case (u, v) => (if(u < 0) ~u else u, v) }
      .partitionBy(new CCPartitioner(numPartitions))
      .mapPartitions(edges => UnionFind.run(edges))

  }

  class CCPartitioner(numParts: Int) extends Partitioner {

    def getPartition(key: Any): Int = {
      (key.asInstanceOf[Long] % numParts).toInt
    }

    override def numPartitions: Int = numParts
  }

}


