package pegasus.spark.cc.deprecated.alpha

import java.io.File
import java.nio.file.{Files, Path, Paths}
import java.util.Random

import pegasus.spark.common.io.{DirectReader, DirectWriter}

import scala.collection.mutable

/**
  * Created by hmpark on 17. 7. 24.
  */
object Consumer {

  val buffer = new Array[Long](5000000) // use 40mb

  def consumeAndReproduce(it: Iterator[Long], basePaths: Array[String]): Iterator[Long] ={

    LocalPaths.setBasePaths(basePaths)

    var queue = new mutable.Queue[Iterator[Long]]()

    var prev: Long = Long.MinValue

    while(it.hasNext) {

      var buffer_filled_size = 0
      while(it.hasNext && buffer_filled_size < buffer.length){

        val curr = it.next()
        if(prev != curr){
          buffer(buffer_filled_size) = curr
          buffer_filled_size += 1
          prev = curr
        }
      }

      if(it.hasNext){
        val tmpPath = LocalPaths.getTempPath("consumer")
        val out = new DirectWriter(tmpPath)

        var i = 0
        while(i < buffer_filled_size){
          out.write(buffer(i))
          i += 1
        }

        out.close()

        queue.enqueue(getDirectReaderIterator(tmpPath))
      }
      else{
        queue.enqueue(getBufferIterator(buffer_filled_size))
      }

    }

    getIterator(queue)

  }

  def getBufferIterator(limit: Int): Iterator[Long] = new Iterator[Long] {
    assert(limit <= buffer.length)
    private var i: Int = 0
    override def hasNext: Boolean = i < limit

    override def next(): Long = {
      val next = buffer(i)
      i += 1
      next
    }
  }

  def getIterator(queue: mutable.Queue[Iterator[Long]]): Iterator[Long] =
    new Iterator[Long] {
      var current: Iterator[Long] = Iterator.empty

      override def hasNext: Boolean = {
        if (!current.hasNext && queue.nonEmpty) {
          current = queue.dequeue()
        }
        return current.hasNext
      }

      override def next(): Long = {
        if (hasNext) {
          current.next()
        }
        else Iterator.empty.next()
      }
    }


  def getDirectReaderIterator(path: String): Iterator[Long] ={

    new Iterator[Long] {
      var dr: DirectReader = _
      override def hasNext: Boolean = dr match {
        case null =>
          dr = new DirectReader(path)
          hasNext
        case _ =>
          dr.hasNext match {
            case true => true
            case false => {dr.close(); Files.delete(Paths.get(path)); false}
          }
      }

      override def next(): Long = if(dr.hasNext) dr.readLong() else Iterator.empty.next()
    }

  }

  private object LocalPaths {
    private val rand: Random = new Random
    private var basePaths: Array[String] = _

    def setBasePaths(basePaths: Array[String]): Unit = {
      for (tmpDir <- basePaths) {
        new File(tmpDir).mkdirs
      }
      LocalPaths.basePaths = basePaths
    }

    private def getNextBasePath: String = basePaths(rand.nextInt(basePaths.length))

    def getTempPath(prefix: String): String = {
      var res: String = null
      do {
        val t: Int = rand.nextInt(Integer.MAX_VALUE)
        res = getNextBasePath + "/" + prefix + "-" + t
      } while (Files.exists(Paths.get(res)))
      res
    }
  }

}
