package pegasus.spark.common

import org.apache.spark.SparkContext
import org.apache.spark.rdd.RDD

object GraphLoader {
  def edgeListFile(
    sc: SparkContext,
    path: String,
    fieldDelimiter: String = "\t"
  ): RDD[(Long, Long)] = {
    sc.textFile(path)
      .mapPartitions(iter => {
        iter.map(line => {
          val point = line.indexOf(fieldDelimiter)
          (line.substring(0, point).toLong, line.substring(point + 1).toLong)
        })
      }, preservesPartitioning = true)
  }

  def weightedEdgeListFile(
    sc: SparkContext,
    path: String,
    fieldDelimiter: String = "\t"
  ): RDD[(Long, Long, Double)] = {
    sc.textFile(path)
      .mapPartitions(iter => {
        iter.map(line => {
          val point1 = line.indexOf(fieldDelimiter)
          val point2 = line.indexOf(fieldDelimiter, point1 + 1)
          val src = line.substring(0, point1).toLong
          val dst = line.substring(point1 + 1, point2).toLong
          val weight = line.substring(point2 + 1).toDouble
          (src, dst, weight)
        })
      })
  }
}
