package pegasus.spark.common.io

import scala.reflect.ClassTag

class FileGroupedIterator[@specialized(Int, Long, Double) T: ClassTag](
  keyPath: String, countPath: String, valueReader: FileReader[T]
) extends Iterator[(Int, Int, Array[T])] with AutoCloseable {
  private val keyReader = new IntFileReader(keyPath)
  private val countReader = new IntFileReader(countPath)

  override def hasNext: Boolean = keyReader.hasNext

  override def next(): (Int, Int, Array[T]) = {
    val key = keyReader.next()
    val count = countReader.next()
    val value = Array.ofDim[T](count)

    var i = 0
    while (i < count) {
      value(i) = valueReader.next()
      i += 1
    }

    (key, count, value)
  }

  override def close(): Unit = {
    keyReader.close()
    countReader.close()
    valueReader.close()
  }
}
