package pegasus.spark.common.io

import java.io.RandomAccessFile
import java.nio.ByteBuffer
import java.nio.channels.FileChannel

import pegasus.spark.common.Constants

import scala.reflect.ClassTag

abstract class FileReader[@specialized(Int, Long, Double) T: ClassTag](
  path: String
) extends Iterator[T] with AutoCloseable {
  private val file = new RandomAccessFile(path, "r")
  protected val buffer: ByteBuffer = try {
    ByteBuffer.allocateDirect(Constants.ByteBufferSize)
  } catch {
    case _: OutOfMemoryError =>
      ByteBuffer.allocate(Constants.ByteBufferSize)
  }
  buffer.flip()

  protected val channel: FileChannel = file.getChannel

  protected def fillBuf(): Unit = {
    buffer.clear()
    channel.read(buffer)
    buffer.flip()
  }

  override def hasNext: Boolean = buffer.hasRemaining || channel.size() > channel.position()

  override def close(): Unit = {
    channel.close()
    file.close()
  }
}
