package pegasus.spark.common.io

import java.io.RandomAccessFile
import java.nio.ByteBuffer
import java.nio.channels.FileChannel
import java.nio.file.{Files, Paths}

import pegasus.spark.common.Constants

import scala.reflect.ClassTag

abstract class FileWriter[@specialized(Int, Long, Double) T: ClassTag](path: String) extends AutoCloseable {
  // remove and initialize file
  Files.deleteIfExists(Paths.get(path))
  private val file = new RandomAccessFile(path, "rw")

  // initialize buffer and direct channel
  protected val buffer: ByteBuffer = try {
    ByteBuffer.allocateDirect(Constants.ByteBufferSize)
  } catch {
    case _: OutOfMemoryError =>
      ByteBuffer.allocate(Constants.ByteBufferSize)
  }
  protected val channel: FileChannel = file.getChannel

  def write(value: T): Unit

  def flush(): Unit = {
    buffer.flip()
    channel.write(buffer)
    buffer.clear()
  }

  override def close(): Unit = {
    flush()
    channel.close()
    file.close()
  }
}
