package pegasus.spark.common.io

import java.io.EOFException
import java.nio.BufferUnderflowException

class LongDoublePairReader(path: String) extends FileReader[(Long, Double)](path) {
  override final def next(): (Long, Double) = (readLong(), readDouble())

  private def readLong(): Long = {
    try {
      buffer.getLong()
    } catch {
      case _: BufferUnderflowException =>
        fillBuf()
        if (buffer.hasRemaining) {
          buffer.getLong()
        } else {
          throw new EOFException()
        }
    }
  }

  private def readDouble(): Double = {
    try {
      buffer.getDouble()
    } catch {
      case _: BufferUnderflowException =>
        fillBuf()
        if (buffer.hasRemaining) {
          buffer.getDouble()
        } else {
          throw new EOFException()
        }
    }
  }
}
