package pegasus.spark.common.io

import com.esotericsoftware.kryo.io.{Input, KryoDataInput}
import org.apache.hadoop.fs.Path
import pegasus.spark.common.PMVUtils

abstract class MatrixReader[T](path: String) extends AutoCloseable {
  private val pathObj = new Path(path)
  private val in = {
    val fs = pathObj.getFileSystem(PMVUtils.hadoopConf())
    val codec = PMVUtils.compressionCodec()
    new Input(codec.compressedInputStream(fs.open(pathObj)))
  }
  protected val dataIn = new KryoDataInput(in)

  final def hasNext: Boolean = !in.eof()

  final def next(): (Int, Int, Array[T]) = {
    val col = dataIn.readInt()
    val degree = dataIn.readInt()
    val rows = readRows()

    (col, degree, rows)
  }

  protected def readRows(): Array[T]

  override def close(): Unit = {
    in.close()
  }
}
