package pegasus.spark.common.io

import com.esotericsoftware.kryo.io.{KryoDataOutput, Output}
import org.apache.hadoop.fs.Path
import pegasus.spark.common.PMVUtils

abstract class MatrixWriter[T](path: String) extends AutoCloseable {
  private val pathObj = new Path(path)
  private val out = {
    val fs = pathObj.getFileSystem(PMVUtils.hadoopConf())
    val codec = PMVUtils.compressionCodec()
    new Output(codec.compressedOutputStream(fs.create(pathObj, 1.toShort)))
  }
  protected val dataOut = new KryoDataOutput(out)

  final def write(col: Int, degree: Int, rows: Array[T]): Unit = {
    dataOut.writeInt(col)
    dataOut.writeInt(degree)
    writeRows(rows)
  }

  protected def writeRows(rows: Array[T]): Unit

  override final def close(): Unit = {
    out.close()
  }
}
