package pegasus.spark.common.io

import com.esotericsoftware.kryo.io.{KryoDataOutput, Output}
import org.apache.hadoop.fs.Path
import pegasus.spark.common.PMVUtils

abstract class VectorWriter[T](path: String) extends AutoCloseable {
  private val pathObj = new Path(path)
  private val out = {
    val fs = pathObj.getFileSystem(PMVUtils.hadoopConf())
    val codec = PMVUtils.compressionCodec()
    new Output(codec.compressedOutputStream(fs.create(pathObj, 1.toShort)))
  }
  protected val dataOut = new KryoDataOutput(out)

  final def write(i: Int, v: T): Unit = {
    dataOut.writeInt(i)
    writeValue(v)
  }

  protected def writeValue(v: T): Unit

  override final def close(): Unit = {
    out.close()
  }
}
