package pegasus.spark.common.io

import pegasus.spark.common.Constants

class WeightedMatrixReader(path: String) extends MatrixReader[(Int, Double)](path) {
  private var rows = Array.ofDim[(Int, Double)](Constants.InitArrayWritableSize)

  override protected def readRows(): Array[(Int, Double)] = {
    val headInt = dataIn.readInt()
    val headDouble = dataIn.readDouble()

    if ((headInt & Int.MinValue) != 0) {
      val length = dataIn.readInt()
      val rowsLocal = if (rows.length < length) {
        rows = Array.ofDim[(Int, Double)](length)
        rows
      } else {
        rows
      }

      var i = 1
      rowsLocal(0) = (headInt & Int.MaxValue, headDouble)
      while (i < length) {
        rowsLocal(i) = (dataIn.readInt(), dataIn.readDouble())
        i += 1
      }
      rowsLocal.slice(0, length)
    } else {
      Array((headInt & Int.MaxValue, headDouble))
    }
  }
}
