package pegasus.spark.common.types

class BitSet(val cardinality: Int) {
  
  import BitSet._

  private val bits = Array.ofDim[Long](math.ceil(cardinality / 64.0).toInt)

  def arraySize: Int = bits.length

  def contains(id: Int): Boolean = {
    val pos = id / 64
    val mask = Masks(id % 64)
    (bits(pos) & mask) != 0
  }

  def add(id: Int): Unit = {
    val pos = id / 64
    val mask = Masks(id % 64)
    bits(pos) |= mask
  }

  def remove(id: Int): Unit = {
    val pos = id / 64
    val mask = Masks(id % 64)
    bits(pos) &= ~mask
  }

  def clear(): Unit = {
    java.util.Arrays.fill(bits, 0L)
  }
}

object BitSet {
  val Masks: Array[Long] = Array.tabulate[Long](64)(i => 1L << i)
}
