package pegasus.spark.common.types

import pegasus.spark.common.Constants

import scala.annotation.tailrec

class BitStrings {
  private val bits = Array.ofDim[Long](Constants.DefaultNumBitstrings)

  def setBits(bits: Array[Long]): Unit = {
    System.arraycopy(bits, 0, this.bits, 0, Constants.DefaultNumBitstrings)
  }

  def getBits: Array[Long] = bits

  def getBit(i: Int) = bits(i)

  def setBit(i: Int, bit: Long): Unit = {
    bits(i) = bit
  }

  def or(other: BitStrings): Unit = {
    var i = 0
    while (i < Constants.DefaultNumBitstrings) {
      bits(i) |= other.bits(i)
      i += 1
    }
  }

  def isEmpty: Boolean = {
    @tailrec
    def recur(i: Int): Boolean = {
      if (i == Constants.DefaultNumBitstrings) true
      else if (bits(i) != 0) false
      else recur(i + 1)
    }

    recur(0)
  }

  def cardinality(): Double = {
    @tailrec
    def zeroBitpos(bit: Long, i: Int): Int = {
      if (i == 64) 64
      else if ((bit & (1L << (63 - i))) == 0) i
      else zeroBitpos(bit, i + 1)
    }

    val bitpos = bits.map(b => zeroBitpos(b, 0)).sum / Constants.DefaultNumBitstrings
    math.pow(2, bitpos) / Constants.Phi
  }


  def canEqual(other: Any): Boolean = other.isInstanceOf[BitStrings]

  override def equals(other: Any): Boolean = other match {
    case that: BitStrings =>
      (that canEqual this) &&
        (bits sameElements that.bits)
    case _ => false
  }

  override def hashCode(): Int = {
    val state = Seq(bits)
    state.map(_.hashCode()).foldLeft(0)((a, b) => 31 * a + b)
  }
}
