package pegasus.spark.common.types

import java.util

import pegasus.spark.common.{Constants, PMVUtils}

import scala.annotation.tailrec

class BitStringsVectorBlock(numBlocks: Int, numVertices: Long, _blockId: Int)
  extends VectorBlock[BitStrings](numBlocks, numVertices, _blockId) {
  private var values: Array[Array[Long]] = _
  private var zeroValue = Array.ofDim[Long](Constants.DefaultNumBitstrings)
  private val value = new BitStrings

  override def get(i: Int): BitStrings = {
    var k = 0
    while (k < Constants.DefaultNumBitstrings) {
      value.setBit(k, values(k)(i))
      k += 1
    }
    value
  }

  override def put(i: Int, v: BitStrings): Unit = {
    mark(i)

    var k = 0
    while (k < Constants.DefaultNumBitstrings) {
      values(k)(i) = value.getBit(k)
      k += 1
    }
  }

  override def addTo(i: Int, v: BitStrings): Unit = {
    mark(i)

    var k = 0
    while (k < Constants.DefaultNumBitstrings) {
      values(k)(i) |= value.getBit(k)
      k += 1
    }
  }

  override def clear(): Unit = {
    var k = 0
    while (k < Constants.DefaultNumBitstrings) {
      util.Arrays.fill(values(k), zeroValue(k))
      k += 1
    }
  }

  override def iterator(onlyNonzero: Boolean) = new VectorIterator[BitStrings] {
    private var currPos = 0
    private var currId = 0
    private val currVal = new BitStrings()

    override final def next(): Boolean = {
      var found = false

      while (!found && currPos < size && PMVUtils.localToGlobal(numBlocks, blockId, currPos) < numVertices) {
        if (!onlyNonzero || !isEmpty) {
          found = true
          currId = currPos

          var k = 0
          while (k < Constants.DefaultNumBitstrings) {
            currVal.setBit(k, values(k)(currPos))
            k += 1
          }
        }
        currPos += 1
      }

      found
    }

    override final def currentId: Int = currId

    override final def currentValue: BitStrings = currVal

    private def isEmpty: Boolean = {
      @tailrec
      def recur(k: Int): Boolean = {
        if (k == Constants.DefaultNumBitstrings) true
        else if (values(k)(currPos) != 0) false
        else recur(k + 1)
      }

      recur(0)
    }
  }

  override def iterator(onlyNonZero: Boolean, filterSet: BitSet) = new VectorIterator[BitStrings] {
    private val iter = iterator(onlyNonZero)

    override final def next(): Boolean = {
      var found = false

      while (!found && iter.next()) {
        found = filterSet.contains(iter.currentId)
      }

      found
    }

    override final def currentId: Int = iter.currentId

    override final def currentValue: BitStrings = iter.currentValue
  }
}
