package pegasus.spark.common.types

import pegasus.spark.common.{Constants, PMVUtils}

class DoubleVectorBlock(numBlocks: Int, numVertices: Long, _blockId: Int, initValue: Double = 0.0)
  extends VectorBlock[Double](numBlocks, numVertices, _blockId) {
  private val values = Array.ofDim[Double](size)
  if (math.abs(initValue) > 0.0) {
    clear()
  }

  override final def get(i: Int) = values(i)

  override final def put(i: Int, v: Double): Unit = {
    mark(i)
    values(i) = v
  }

  override final def addTo(i: Int, v: Double): Unit = {
    mark(i)
    values(i) += v
  }

  override final def clear(): Unit = {
    java.util.Arrays.fill(values, initValue)
  }

  override final def iterator(onlyNonzero: Boolean) = new VectorIterator[Double] {
    private var currPos = 0
    private var currId = 0
    private var currVal = 0.0

    override final def next(): Boolean = {
      var found = false

      while (!found && currPos < size && PMVUtils.localToGlobal(numBlocks, blockId, currPos) < numVertices) {
        if (!onlyNonzero || math.abs(values(currPos)) > Constants.DoubleNonzeroThreshold) {
          found = true
          currId = currPos
          currVal = values(currPos)
        }
        currPos += 1
      }

      found
    }

    override final def currentId: Int = currId

    override final def currentValue: Double = currVal
  }

  override final def iterator(onlyNonZero: Boolean, filterSet: BitSet) = new VectorIterator[Double] {
    private val iter = iterator(onlyNonZero)

    override final def next(): Boolean = {
      var found = false

      while (!found && iter.next()) {
        found = filterSet.contains(iter.currentId)
      }

      found
    }

    override final def currentId: Int = iter.currentId

    override final def currentValue: Double = iter.currentValue
  }
}
