package pegasus.spark.common.types

abstract class VectorBlock[@specialized(Int, Long, Double) T](
  numBlocks: Int,
  numVertices: Long,
  private var _blockId: Int
) {
  val size: Int = math.ceil(numVertices / numBlocks.toDouble).toInt
  private val containsSet = new BitSet(size)

  final def contains(i: Int): Boolean = containsSet.contains(i)

  protected final def mark(i: Int): Unit = containsSet.add(i)

  protected final def clearContainsSet(): Unit = containsSet.clear()

  def blockId: Int = _blockId

  def setBlockId(blockId: Int): Unit = {
    _blockId = blockId
  }

  def get(i: Int): T

  def put(i: Int, v: T): Unit

  def addTo(i: Int, v: T): Unit

  def clear(): Unit

  def iterator(onlyNonzero: Boolean): VectorIterator[T]

  def iterator(onlyNonZero: Boolean, filterSet: BitSet): VectorIterator[T]
}
