package pegasus.spark.common.types

class WeightedMatrixBlock(
  override val blockId: Int,
  override val numCols: Int,
  override val numRows: Int
) extends MatrixBlock[(Int, Double)] {
  private var lastCol = 0
  private var lastRow = 0
  private val cols = Array.ofDim[Int](numCols)
  private val degs = Array.ofDim[Int](numCols)
  private val rows = Array.ofDim[Int](numRows)
  private val rowSizes = Array.ofDim[Int](numCols)
  private val weights = Array.ofDim[Double](numRows)

  private[WeightedMatrixBlock] def setMetadata(lastCol: Int, lastRow: Int): Unit = {
    this.lastCol = lastCol
    this.lastRow = lastRow
  }

  private[WeightedMatrixBlock] def getArrays = (cols, degs, rows, rowSizes)

  override final def +=(col: Int, deg: Int, rows: Array[(Int, Double)]): Unit = {
    cols(lastCol) = col
    degs(lastCol) = deg
    var i = 0
    while (i < rows.length) {
      this.rows(lastRow + i) = rows(i)._1
      this.weights(lastRow + i) = rows(i)._2
      i += 1
    }
    lastRow += rows.length
    rowSizes(lastCol) = lastRow
    lastCol += 1
  }

  override final def iterator(): MatrixIterator[(Int, Double)] = {
    new MatrixIterator[(Int, Double)] {
      private var currPos = 0
      private var currCol = 0
      private var currDeg = 0
      private val currRows = new WeightedMatrixRows()

      override def next(): Boolean = {
        if (currPos < lastCol) {
          currCol = cols(currPos)
          currDeg = degs(currPos)

          val begin = if (currPos == 0) 0 else rowSizes(currPos - 1)
          val end = rowSizes(currPos)
          currRows.setRows(rows, weights, begin, end)

          currPos += 1
          true
        } else {
          false
        }
      }

      override def currentRows(): MatrixRows[(Int, Double)] = currRows

      override def currentCol(): Int = currCol

      override def currentDeg(): Int = currDeg
    }
  }
}
