package pegasus.spark.subgraph

import java.util.StringTokenizer

import org.apache.hadoop.fs.{FileSystem, Path}
import org.apache.log4j.{Level, Logger}
import org.apache.spark.{SparkConf, SparkContext}

/**
  * Created by hmpark on 17. 3. 14.
  */
object EdgeOrder {

  Logger.getLogger("org").setLevel(Level.OFF)
  Logger.getLogger("akka").setLevel(Level.OFF)

  val logger = Logger.getLogger(EdgeOrder.getClass)

  def main(args: Array[String]): Unit = {
    val inputPath = args(0)
    val outputPath = args(1)

    val local = if(args.length >= 3) args(2).toBoolean else false

    logger.info(f"inputPath: $inputPath, output: $outputPath")

    run(inputPath, outputPath, local)

  }

  def run(inputPath: String, outputPath: String, local: Boolean): Unit = {

    var time_startup = System.currentTimeMillis()

    val conf = new SparkConf().setAppName("[EdgeOrder]" + inputPath) //.setMaster("local")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .set("spark.kryo.registrator", "pegasus.spark.subgraph.MyRegistrator")
      .set("spark.kryoserializer.buffer.mb", "24")

    if(local) conf.setMaster("local[2]")

    val sc = new SparkContext(conf)

    val fs = FileSystem.get(sc.hadoopConfiguration)

    //find numNodes
    sc.textFile(inputPath).flatMap { line =>
      val st = new StringTokenizer(line)
      val u = st.nextToken().toLong
      val v = st.nextToken().toLong

      if ( u != v ) Seq((u, v), (v, u))
      else Seq()

    }.groupByKey().flatMap {
      case (u, _uN) =>
        val uN = _uN.toArray.sorted

        val uDeg = uN.foldLeft((-1L, 0)){case ((prev, uDeg), v) => (v, if(prev != v) uDeg + 1 else uDeg) }._2

        var prev = -1L
        uN.filter{v => val res = v != prev; prev = v; res}.map{v => ((u, v), uDeg)}
    }.map{
      case ((u, v), uDeg) =>
        if( u < v ) ((u,v), uDeg)
        else ((v, u), -uDeg)
    }.groupByKey().map{
      case ((u, v), degs) =>

        val it = degs.iterator
        val tmp = it.next
        val (uDeg, vDeg) = if (tmp < 0) (it.next, -tmp)
                           else (tmp, -it.next)

        if (uDeg < vDeg || (uDeg == vDeg && u < v))
          (u, v)
        else
          (v, u)
    }.saveAsSequenceFile(outputPath)

    sc.stop()
  }


}
