package pegasus.spark.cc.deprecated.alpha

import org.apache.hadoop.fs.{FileSystem, Path}
import org.apache.log4j.{Level, Logger}
import org.apache.spark.{SparkConf, SparkContext}
import org.scalatest.{FlatSpec, Matchers}

/**
  * Created by hmpark on 17. 3. 17.
  */
class PACCSortSpec extends FlatSpec with Matchers {

  val logger = Logger.getLogger(getClass)
  logger.setLevel(Level.INFO)

  "largeStar" should "work correctly on an example data" in {

    val conf = new SparkConf().setAppName("largeStarTest").setMaster("local[1]")

    val sc = new SparkContext(conf)

    val inputdata = Seq[(Long, Long)]((~2, 1), (~3, 1), (~4, 1), (6, 5), (7,6), (8,6), (9,6), (9,7))
    val inputRDD = sc.parallelize(inputdata).repartition(4)


    val (lout, numChanges, numOut, numCC, numIn) = PACCSort.largeStar(inputRDD, 2, 1, "tmp_res")

    lout.collect().toSet should be (Set((6,5), (7,5), (9,5), (9,7), (8,6)))

    sc.sequenceFile[Long, Long]("tmp_res")
      .collect().toSet.equals(Set((2,1), (3,1), (4,1))) should be (true)

    FileSystem.get(sc.hadoopConfiguration).delete(new Path("tmp_res"), true)

    numChanges should be (2)
    numOut should be (5)
    numIn should be (0)
    numCC should be (3)

    println(f"num changes: $numChanges, numOut: $numOut, numCC: $numCC, numIn: $numIn")

    sc.stop()

  }

  "smallStar" should "work correctly on an example data" in {

    val conf = new SparkConf().setAppName("smallStarTest").setMaster("local[2]")

    val sc = new SparkContext(conf)

    val inputdata = Seq[(Long, Long)]((1,6), (2,6), (4,6), (2,5), (2,3), (5,7))
    val inputRDD = sc.parallelize(inputdata).repartition(4)

    val (sout, numChanges, numOut, numIn) = PACCSort.smallStar(inputRDD, 2, 1, "tmp_res")

    Set((2,1),(4,2),(~3,2),(5,2)).equals(sout.collect().toSet) should be (true)

    sc.sequenceFile[Long, Long]("tmp_res")
    .collect().toSet.equals(Set((6,2),(7,5))) should be (true)

    numChanges should be (2)
    numOut should be (4)
    numIn should be (2)

    FileSystem.get(sc.hadoopConfiguration).delete(new Path("tmp_res"), true)

    println(f"num changes: $numChanges, numOut: $numOut, numIn: $numIn")

    sc.stop()
  }

  "a combination of largeStar and smallStar" should "work correctly on an example data" in {

    val conf = new SparkConf().setAppName("largeStarTest").setMaster("local[1]")

    val sc = new SparkContext(conf)

    val inputdata = Seq[(Long, Long)]((~2, 1), (~3, 1), (~4, 1), (6, 5), (7,6), (8,6), (9,6), (9,7))
    val inputRDD = sc.parallelize(inputdata).repartition(4)

    val (lout, numChanges1, numOut1, numCC1, numIn1) = PACCSort.largeStar(inputRDD, 2, 1, "tmp_res")

    numChanges1 should be (2)
    numOut1 should be (5)
    numIn1 should be (0)
    numCC1 should be (3)

    lout.collect().toSet should be (Set((6,5), (7,5), (9,5), (9,7), (8,6)))

    val (sout, numChanges2, numOut2, numIn2) = PACCSort.smallStar(lout, 2, 1, "tmp_res")

    numChanges2 should be (1)
    numOut2 should be (3)
    numIn2 should be (2)

    sc.sequenceFile[Long, Long]("tmp_res")
      .collect().toSet should contain only ((2,1), (3,1), (4,1), (9,5), (8,6))

    FileSystem.get(sc.hadoopConfiguration).delete(new Path("tmp_res"), true)

    sc.stop()

  }

  "PACCSort" should "output 4157 nodes from GCC in grqc" in {
    val path = getClass.getResource("/graphs/grqc").toString
    val lt = 0
    val np = 10

    logger.info(f"test( data: $path, localThreahold: $lt, numPartition: $np )")

    val conf = new SparkConf().setAppName("PACCSort-test").setMaster("local[2]")
      .set("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
      .set("spark.kryo.registrator", "pegasus.spark.subgraph.MyRegistrator")
      .set("spark.kryoserializer.buffer", "24m")

    val sc = new SparkContext(conf)
    val numNodesInGCC = PACCSort.run(path, lt, np, sc)
      .map{case (u, c) => (c, u)}
      .groupByKey()
      .map{case (c, nodes) => nodes.size}
      .collect().max

    numNodesInGCC should be (4157)
  }

}
